@@ -19,15 +19,6 @@
  *  @ingroup    hard_constraints
  *  @brief      Functions and data structures for handling of secondary structure hard constraints
  */
-#include <ViennaRNA/fold_compound.h>
-#include <ViennaRNA/constraints/basic.h>
-
-/**
- *  @addtogroup hard_constraints
- *
- *  @brief  This module covers all functionality for hard constraints in secondary
- *          structure prediction
- */
 
 /**
  *  @brief Typename for the hard constraints data structure #vrna_hc_s
@@ -41,6 +32,12 @@ typedef struct  vrna_hc_s vrna_hc_t;
  */
 typedef struct vrna_hc_up_s vrna_hc_up_t;
 
+typedef struct vrna_hc_depot_s vrna_hc_depot_t;
+
+#include <ViennaRNA/fold_compound.h>
+#include <ViennaRNA/constraints/basic.h>
+#include <ViennaRNA/datastructures/sparse_mx.h>
+
 /**
  * @brief Callback to evaluate whether or not a particular decomposition step is contributing to the solution space
  *
@@ -72,12 +69,38 @@ typedef struct vrna_hc_up_s vrna_hc_up_t;
  * @param data      Auxiliary data
  * @return          A non-zero value if the decomposition is valid, 0 otherwise
  */
-typedef unsigned char (vrna_callback_hc_evaluate)(int           i,
-                                                  int           j,
-                                                  int           k,
-                                                  int           l,
-                                                  unsigned char d,
-                                                  void          *data);
+typedef unsigned char (*vrna_hc_eval_f)(int           i,
+                                        int           j,
+                                        int           k,
+                                        int           l,
+                                        unsigned char d,
+                                        void          *data);
+
+typedef unsigned char (*vrna_hc_eval_loop_f)(int           i,
+                                             int           j,
+                                             int           k,
+                                             int           l,
+                                             unsigned char d,
+                                             vrna_hc_t     *hc);
+
+typedef unsigned char (*vrna_hc_eval_hp_f)(int           i,
+                                           int           j,
+                                           vrna_hc_t     *hc);
+
+typedef unsigned char (*vrna_hc_eval_int_f)(int           i,
+                                            int           j,
+                                            int           k,
+                                            int           l,
+                                            vrna_hc_t     *hc);
+
+DEPRECATED(typedef unsigned char (vrna_callback_hc_evaluate)(int i,
+                                                             int j,
+                                                             int k,
+                                                             int l,
+                                                             unsigned char d,
+                                                             void *data),
+           "Use vrna_hc_eval_f instead!");
+
 
 /**
  *  @brief  do not print the header information line
@@ -231,13 +254,19 @@ typedef unsigned char (vrna_callback_hc_evaluate)(int           i,
    | VRNA_CONSTRAINT_DB_GQUAD \
   )
 
+
+/**
+ *  @brief  Constraint context flag that forbids a nucleotide or base pair to appear in any loop
+ */
+#define VRNA_CONSTRAINT_CONTEXT_NONE          (unsigned char)0
+
 /**
  *  @brief  Hard constraints flag, base pair in the exterior loop
  *
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_EXT_LOOP      (unsigned char)0x01
+#define VRNA_CONSTRAINT_CONTEXT_EXT_LOOP      (unsigned char)(1 << 0)
 
 /**
  *  @brief  Hard constraints flag, base pair encloses hairpin loop
@@ -245,15 +274,15 @@ typedef unsigned char (vrna_callback_hc_evaluate)(int           i,
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_HP_LOOP       (unsigned char)0x02
+#define VRNA_CONSTRAINT_CONTEXT_HP_LOOP       (unsigned char)(1 << 1)
 
 /**
- *  @brief  Hard constraints flag, base pair encloses an interior loop
+ *  @brief  Hard constraints flag, base pair encloses an internal loop
  *
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_INT_LOOP      (unsigned char)0x04
+#define VRNA_CONSTRAINT_CONTEXT_INT_LOOP      (unsigned char)(1 << 2)
 
 /**
  *  @brief  Hard constraints flag, base pair encloses a multi branch loop
@@ -261,15 +290,15 @@ typedef unsigned char (vrna_callback_hc_evaluate)(int           i,
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC  (unsigned char)0x08
+#define VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC  (unsigned char)(1 << 3)
 
 /**
- *  @brief  Hard constraints flag, base pair is enclosed in an interior loop
+ *  @brief  Hard constraints flag, base pair is enclosed in an internal loop
  *
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_MB_LOOP       (unsigned char)0x10
+#define VRNA_CONSTRAINT_CONTEXT_MB_LOOP       (unsigned char)(1 << 4)
 
 /**
  *  @brief  Hard constraints flag, base pair is enclosed in a multi branch loop
@@ -277,23 +306,17 @@ typedef unsigned char (vrna_callback_hc_evaluate)(int           i,
  *  @ingroup  hard_constraints
  *
  */
-#define VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC   (unsigned char)0x20
+#define VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC   (unsigned char)(1 << 5)
 
 /**
  *  @brief  Hard constraint flag to indicate enforcement of constraints
  */
-#define VRNA_CONSTRAINT_CONTEXT_ENFORCE       (unsigned char)0x40
+#define VRNA_CONSTRAINT_CONTEXT_ENFORCE       (unsigned char)(1 << 6)
 
 /**
  *  @brief  Hard constraint flag to indicate not to remove base pairs that conflict with a given constraint
  */
-#define VRNA_CONSTRAINT_CONTEXT_NO_REMOVE     (unsigned char)0x80
-
-
-/**
- *  @brief  Constraint context flag that forbids any loop
- */
-#define VRNA_CONSTRAINT_CONTEXT_NONE          (unsigned char)0
+#define VRNA_CONSTRAINT_CONTEXT_NO_REMOVE     (unsigned char)(1 << 7)
 
 /**
  *  @brief  Constraint context flag indicating base pairs that close any loop
@@ -334,16 +357,6 @@ typedef enum {
 } vrna_hc_type_e;
 
 
-/**
- *  @brief  A base pair hard constraint
- */
-typedef struct {
-  int           interval_start;
-  int           interval_end;
-  unsigned char loop_type;
-  unsigned char replace : 1;
-} vrna_hc_bp_storage_t;
-
 /**
  *  @brief  The hard constraints data structure
  *
@@ -354,7 +367,7 @@ typedef struct {
  *  following types of base pairs:
  *  - in the exterior loop (#VRNA_CONSTRAINT_CONTEXT_EXT_LOOP)
  *  - enclosing a hairpin (#VRNA_CONSTRAINT_CONTEXT_HP_LOOP)
- *  - enclosing an interior loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP)
+ *  - enclosing an internal loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP)
  *  - enclosed by an exterior loop (#VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC)
  *  - enclosing a multi branch loop (#VRNA_CONSTRAINT_CONTEXT_MB_LOOP)
  *  - enclosed by a multi branch loop (#VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC)
@@ -369,62 +382,55 @@ typedef struct {
  *  @ingroup hard_constraints
  */
 struct vrna_hc_s {
-  vrna_hc_type_e  type;
-  unsigned int    n;
+  vrna_hc_type_e      type;
+  unsigned int        n;
+  unsigned int        strands;
+  unsigned int        *sn;
 
-#ifndef VRNA_DISABLE_C11_FEATURES
-  /* C11 support for unnamed unions/structs */
-  union {
-    struct {
-#endif
-      unsigned char *matrix;     /**<  @brief  Upper triangular matrix that encodes where a
-                                  *            base pair or unpaired nucleotide is allowed
-                                  */
-      unsigned char *mx;
-#ifndef VRNA_DISABLE_C11_FEATURES
-    };
-    struct {
-#endif
-      unsigned char         **matrix_local;
-      unsigned char         *up_storage;
-      vrna_hc_bp_storage_t  **bp_storage;
-#ifndef VRNA_DISABLE_C11_FEATURES
-    };
-  };
-#endif
+  unsigned char       state;
 
-  int                         *up_ext;    /**<  @brief  A linear array that holds the number of allowed
-                                           *            unpaired nucleotides in an exterior loop
-                                           */
-  int                         *up_hp;     /**<  @brief  A linear array that holds the number of allowed
-                                           *            unpaired nucleotides in a hairpin loop
-                                           */
-  int                         *up_int;    /**<  @brief  A linear array that holds the number of allowed
-                                           *            unpaired nucleotides in an interior loop
-                                           */
-  int                         *up_ml;     /**<  @brief  A linear array that holds the number of allowed
-                                           *            unpaired nucleotides in a multi branched loop
-                                           */
+  unsigned char       *mx;
+  unsigned char       **matrix_local;
 
-  vrna_callback_hc_evaluate   *f;         /**<  @brief  A function pointer that returns whether or
-                                           *            not a certain decomposition may be evaluated
-                                           */
+  unsigned int        *up_ext;    /**<  @brief  A linear array that holds the number of allowed
+                                   *            unpaired nucleotides in an exterior loop
+                                   */
+  unsigned int        *up_hp;     /**<  @brief  A linear array that holds the number of allowed
+                                   *            unpaired nucleotides in a hairpin loop
+                                   */
+  unsigned int        *up_int;    /**<  @brief  A linear array that holds the number of allowed
+                                   *            unpaired nucleotides in an internal loop
+                                   */
+  unsigned int        *up_ml;     /**<  @brief  A linear array that holds the number of allowed
+                                   *            unpaired nucleotides in a multi branched loop
+                                   */
 
-  void                        *data;      /**<  @brief  A pointer to some structure where the user
-                                           *            may store necessary data to evaluate its
-                                           *            generic hard constraint function
-                                           */
+  vrna_hc_eval_f      f;          /**<  @brief  A function pointer that returns whether or
+                                   *            not a certain decomposition may be evaluated
+                                   */
 
-  vrna_callback_free_auxdata  *free_data; /**<  @brief  A pointer to a function to free memory
-                                           *            occupied by auxiliary data
-                                           *
-                                           *    The function this pointer is pointing to will be
-                                           *    called upon destruction of the #vrna_hc_s, and
-                                           *    provided with the vrna_hc_s.data pointer that
-                                           *    may hold auxiliary data. Hence, to avoid leaking
-                                           *    memory, the user may use this pointer to free
-                                           *    memory occupied by auxiliary data.
-                                           */
+  void                *data;      /**<  @brief  A pointer to some structure where the user
+                                   *            may store necessary data to evaluate its
+                                   *            generic hard constraint function
+                                   */
+
+  vrna_auxdata_free_f free_data;  /**<  @brief  A pointer to a function to free memory
+                                   *            occupied by auxiliary data
+                                   *
+                                   *    The function this pointer is pointing to will be
+                                   *    called upon destruction of the #vrna_hc_s, and
+                                   *    provided with the vrna_hc_s.data pointer that
+                                   *    may hold auxiliary data. Hence, to avoid leaking
+                                   *    memory, the user may use this pointer to free
+                                   *    memory occupied by auxiliary data.
+                                   */
+
+  vrna_hc_depot_t *depot;
+
+  vrna_hc_eval_loop_f eval_ext;
+  vrna_hc_eval_hp_f   eval_hp;
+  vrna_hc_eval_int_f  eval_int;
+  vrna_hc_eval_loop_f eval_mb;
 };
 
 /**
@@ -434,6 +440,7 @@ struct vrna_hc_s {
  */
 struct vrna_hc_up_s {
   int position;           /**<  @brief The sequence position (1-based)  */
+  int strand;
   unsigned char options;  /**<  @brief The hard constraint option       */
 };
 
@@ -459,7 +466,8 @@ struct vrna_hc_up_s {
  *
  *  @param option Option switch that tells which constraint help will be printed
  */
-void vrna_message_constraint_options(unsigned int option);
+void
+vrna_message_constraint_options(unsigned int option);
 
 
 /**
@@ -472,7 +480,8 @@ void vrna_message_constraint_options(unsigned int option);
  *        #VRNA_CONSTRAINT_DB_PIPE, #VRNA_CONSTRAINT_DB_DOT, #VRNA_CONSTRAINT_DB_X, #VRNA_CONSTRAINT_DB_ANG_BRACK,
  *        #VRNA_CONSTRAINT_DB_RND_BRACK, #VRNA_CONSTRAINT_DB_INTERMOL, #VRNA_CONSTRAINT_DB_INTRAMOL
  */
-void vrna_message_constraint_options_all(void);
+void
+vrna_message_constraint_options_all(void);
 
 
 /**
@@ -487,17 +496,25 @@ void vrna_message_constraint_options_all(void);
  *
  *  @see  vrna_hc_add_bp(), vrna_hc_add_bp_nonspecific(), vrna_hc_add_up()
  *
- *  @param  vc  The fold compound
+ *  @param  fc  The fold compound
  */
-void vrna_hc_init(vrna_fold_compound_t *vc);
+void
+vrna_hc_init(vrna_fold_compound_t *fc);
 
 
-void vrna_hc_init_window(vrna_fold_compound_t *vc);
+void
+vrna_hc_init_window(vrna_fold_compound_t *fc);
+
+
+int
+vrna_hc_prepare(vrna_fold_compound_t  *fc,
+                unsigned int          options);
 
 
 void
 vrna_hc_update(vrna_fold_compound_t *fc,
-               unsigned int         i);
+               unsigned int         i,
+               unsigned int         options);
 
 
 /**
@@ -510,13 +527,46 @@ vrna_hc_update(vrna_fold_compound_t *fc,
  *        #VRNA_CONSTRAINT_CONTEXT_INT_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP,
  *        #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
  *
- *  @param  vc      The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  fc      The #vrna_fold_compound_t the hard constraints are associated with
  *  @param  i       The position that needs to stay unpaired (1-based)
  *  @param  option  The options flag indicating how/where to store the hard constraints
  */
-void vrna_hc_add_up(vrna_fold_compound_t  *vc,
-                    int                   i,
-                    unsigned char         option);
+void
+vrna_hc_add_up(vrna_fold_compound_t *fc,
+               unsigned int         i,
+               unsigned char        option);
+
+
+/**
+ *  @brief  Make a certain nucleotide unpaired
+ *
+ *  This function puts a constraint onto a single nucleotide to limit or enforce
+ *  its structural context. The position @p i should be given in local coordinates
+ *  relative to the strand @p strand it corresponds to.
+ *
+ *  @note
+ *  A negative value for the strand number @p strand indicates autodetection
+ *  of the strand number assuming that coordinate i is given as global coordinates
+ *  for the (current) concatenation of all strands. In this case, the function
+ *  behaves exactly as vrna_hc_add_up().
+ *
+ *  @ingroup  hard_constraints
+ *
+ *  @see  vrna_hc_add_bp(), vrna_hc_add_bp_nonspecific(), vrna_hc_init(),
+ *        #VRNA_CONSTRAINT_CONTEXT_EXT_LOOP, #VRNA_CONSTRAINT_CONTEXT_HP_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_INT_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+ *
+ *  @param  fc      The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  i       The position that needs to stay unpaired (1-based)
+ *  @param  strand  The strand number of nucleotide @p i (0-based, negative value for autodetect)
+ *  @param  option  The options flag indicating how/where to store the hard constraints
+ */
+int
+vrna_hc_add_up_strand(vrna_fold_compound_t  *fc,
+                      unsigned int          i,
+                      int                   strand,
+                      unsigned char         option);
 
 
 /**
@@ -524,15 +574,20 @@ void vrna_hc_add_up(vrna_fold_compound_t  *vc,
  *
  *  @ingroup  hard_constraints
  *
- *  @param  vc          The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  fc          The #vrna_fold_compound_t the hard constraints are associated with
  *  @param  constraints The list off constraints to apply, last entry must have position
  *                      attribute set to 0
  */
 int
-vrna_hc_add_up_batch(vrna_fold_compound_t *vc,
+vrna_hc_add_up_batch(vrna_fold_compound_t *fc,
                      vrna_hc_up_t         *constraints);
 
 
+int
+vrna_hc_add_up_strand_batch(vrna_fold_compound_t  *fc,
+                            vrna_hc_up_t          *constraints);
+
+
 /**
  *  @brief  Favorize/Enforce  a certain base pair (i,j)
  *
@@ -544,15 +599,55 @@ vrna_hc_add_up_batch(vrna_fold_compound_t *vc,
  *        #VRNA_CONSTRAINT_CONTEXT_MB_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC,
  *        #VRNA_CONSTRAINT_CONTEXT_ENFORCE, #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
  *
- *  @param  vc      The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  fc      The #vrna_fold_compound_t the hard constraints are associated with
  *  @param  i       The 5' located nucleotide position of the base pair (1-based)
  *  @param  j       The 3' located nucleotide position of the base pair (1-based)
  *  @param  option  The options flag indicating how/where to store the hard constraints
  */
-void vrna_hc_add_bp(vrna_fold_compound_t  *vc,
-                    int                   i,
-                    int                   j,
-                    unsigned char         option);
+int
+vrna_hc_add_bp(vrna_fold_compound_t *fc,
+               unsigned int         i,
+               unsigned int         j,
+               unsigned char        option);
+
+
+/**
+ *  @brief  Favorize/Enforce  a certain base pair (i,j) where i and j may point to different strands
+ *
+ *  This function adds a base pair constraint for pair (i,j), where the positions i and j are
+ *  relative to the RNA strands i and j correspond to. For each strand @p strand_i and @p strand_j
+ *  positions are 1-based. For instance, if position 5 of strand 0 must pair with position 10
+ *  of strand 1, the function call would be
+ *  @code
+ *  vrna_hc_add_bp(fc, 5, 10, 0, 1, VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS | VRNA_CONSTRAINT_CONTEXT_ENFORCE);
+ *  @endcode
+ *
+ *  Negative values for the strand numbers @p strand_i and @p strand_j indicate autodetection
+ *  of the strand number assuming that coordinates i and/or j are given as global coordinates
+ *  for the (current) concatenation of all strands
+ *
+ *  @ingroup  hard_constraints
+ *
+ *  @see  vrna_hc_add_bp_nonspecific(), vrna_hc_add_up(), vrna_hc_add_bp(), vrna_hc_init(),
+ *        #VRNA_CONSTRAINT_CONTEXT_EXT_LOOP, #VRNA_CONSTRAINT_CONTEXT_HP_LOOP,
+ *        #VRNA_CONSTRAINT_CONTEXT_INT_LOOP, #VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC,
+ *        #VRNA_CONSTRAINT_CONTEXT_MB_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC,
+ *        #VRNA_CONSTRAINT_CONTEXT_ENFORCE, #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
+ *
+ *  @param  fc        The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  i         The 5' located nucleotide position of the base pair (1-based, relative to @p strand_i)
+ *  @param  j         The 3' located nucleotide position of the base pair (1-based, relative to @p strand_j)
+ *  @param  strand_i  The strand number of pairing partner @p i (0-based, negative value for autodetect)
+ *  @param  strand_j  The strand number of pairing partner @p j (0-based, negative value for autodetect)
+ *  @param  option    The option flag(s) indicating loop types and enforcement of the constraint
+ */
+int
+vrna_hc_add_bp_strand(vrna_fold_compound_t  *fc,
+                      unsigned int          i,
+                      unsigned int          j,
+                      int                   strand_i,
+                      int                   strand_j,
+                      unsigned char         option);
 
 
 /**
@@ -566,16 +661,17 @@ void vrna_hc_add_bp(vrna_fold_compound_t  *vc,
  *        #VRNA_CONSTRAINT_CONTEXT_MB_LOOP, #VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC,
  *        #VRNA_CONSTRAINT_CONTEXT_ALL_LOOPS
  *
- *  @param  vc      The #vrna_fold_compound_t the hard constraints are associated with
+ *  @param  fc      The #vrna_fold_compound_t the hard constraints are associated with
  *  @param  i       The position that needs to stay unpaired (1-based)
  *  @param  d       The direction of base pairing (@f$ d < 0 @f$: pairs upstream,
  *                  @f$ d > 0 @f$: pairs downstream, @f$ d == 0 @f$: no direction)
  *  @param  option  The options flag indicating in which loop type context the pairs may appear
  */
-void vrna_hc_add_bp_nonspecific(vrna_fold_compound_t  *vc,
-                                int                   i,
-                                int                   d,
-                                unsigned char         option);
+void
+vrna_hc_add_bp_nonspecific(vrna_fold_compound_t *fc,
+                           unsigned int         i,
+                           int                  d,
+                           unsigned char        option);
 
 
 /**
@@ -589,31 +685,34 @@ void vrna_hc_add_bp_nonspecific(vrna_fold_compound_t  *vc,
  *  @ingroup  hard_constraints
  *
  */
-void vrna_hc_free(vrna_hc_t *hc);
+void
+vrna_hc_free(vrna_hc_t *hc);
 
 
 /**
  *  @brief  Add a function pointer pointer for the generic hard constraint
  *          feature
  */
-void vrna_hc_add_f(vrna_fold_compound_t       *vc,
-                   vrna_callback_hc_evaluate  *f);
+void
+vrna_hc_add_f(vrna_fold_compound_t  *fc,
+              vrna_hc_eval_f        f);
 
 
 /**
  *  @brief Add an auxiliary data structure for the generic hard constraints callback function
  *
- *  @ingroup generic_hc
+ *  @ingroup hard_constraints
  *
  *  @see vrna_hc_add_f()
  *
- *  @param  vc        The fold compound the generic hard constraint function should be bound to
+ *  @param  fc        The fold compound the generic hard constraint function should be bound to
  *  @param  data      A pointer to the data structure that holds required data for function 'f'
  *  @param  f         A pointer to a function that free's the memory occupied by @p data (Maybe @p NULL)
  */
-void vrna_hc_add_data(vrna_fold_compound_t        *vc,
-                      void                        *data,
-                      vrna_callback_free_auxdata  *f);
+void
+vrna_hc_add_data(vrna_fold_compound_t *fc,
+                 void                 *data,
+                 vrna_auxdata_free_f  f);
 
 
 /**
@@ -630,16 +729,20 @@ void vrna_hc_add_data(vrna_fold_compound_t        *vc,
  *        #VRNA_CONSTRAINT_DB_ANG_BRACK, #VRNA_CONSTRAINT_DB_RND_BRACK, #VRNA_CONSTRAINT_DB_INTRAMOL,
  *        #VRNA_CONSTRAINT_DB_INTERMOL, #VRNA_CONSTRAINT_DB_GQUAD
  *
- *  @param  vc            The fold compound
+ *  @param  fc            The fold compound
  *  @param  constraint    A pseudo dot-bracket notation of the hard constraint.
  *  @param  options       The option flags
  */
 int
-vrna_hc_add_from_db(vrna_fold_compound_t  *vc,
+vrna_hc_add_from_db(vrna_fold_compound_t  *fc,
                     const char            *constraint,
                     unsigned int          options);
 
 
+vrna_smx_csr(vrna_uchar) *
+vrna_hc_nondefaults(vrna_fold_compound_t *fc);
+
+
 #ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
 
 /**
@@ -649,8 +752,9 @@ vrna_hc_add_from_db(vrna_fold_compound_t  *vc,
  *  @deprecated Use vrna_message_constraints() instead!
  *  @param option Option switch that tells which constraint help will be printed
  */
-DEPRECATED(void print_tty_constraint(unsigned int option),
-          "Use vrna_message_constraint_options() instead");
+DEPRECATED(void
+           print_tty_constraint(unsigned int option),
+           "Use vrna_message_constraint_options() instead");
 
 /**
  *  @brief Print structure constraint characters to stdout
@@ -658,8 +762,9 @@ DEPRECATED(void print_tty_constraint(unsigned int option),
  *
  *  @deprecated Use vrna_message_constraint_options_all() instead!
  */
-DEPRECATED(void print_tty_constraint_full(void),
-          "Use vrna_message_constraint_options_all() instead");
+DEPRECATED(void
+           print_tty_constraint_full(void),
+           "Use vrna_message_constraint_options_all() instead");
 
 /**
  *  @brief Insert constraining pair types according to constraint structure string
@@ -673,13 +778,14 @@ DEPRECATED(void print_tty_constraint_full(void),
  *  @param min_loop_size  The minimal loop size (usually #TURN )
  *  @param idx_type       Define the access type for base pair type array (0 = indx, 1 = iindx)
  */
-DEPRECATED(void constrain_ptypes(const char   *constraint,
-                                 unsigned int length,
-                                 char         *ptype,
-                                 int          *BP,
-                                 int          min_loop_size,
-                                 unsigned int idx_type),
-          "Use the new API and the hard constraint framework instead");
+DEPRECATED(void
+           constrain_ptypes(const char    *constraint,
+                            unsigned int  length,
+                            char          *ptype,
+                            int           *BP,
+                            int           min_loop_size,
+                            unsigned int  idx_type),
+           "Use the new API and the hard constraint framework instead");
 
 #endif
 
