@@ -1,6 +1,18 @@
 #ifndef VIENNA_RNA_PACKAGE_FOLD_COMPOUND_H
 #define VIENNA_RNA_PACKAGE_FOLD_COMPOUND_H
 
+#ifdef VRNA_WARN_DEPRECATED
+# if defined(__clang__)
+#  define DEPRECATED(func, msg) func __attribute__ ((deprecated("", msg)))
+# elif defined(__GNUC__)
+#  define DEPRECATED(func, msg) func __attribute__ ((deprecated(msg)))
+# else
+#  define DEPRECATED(func, msg) func
+# endif
+#else
+# define DEPRECATED(func, msg) func
+#endif
+
 /**
  *  @file     fold_compound.h
  *  @ingroup  fold_compound
@@ -10,14 +22,6 @@
 /**
  *  @addtogroup   fold_compound   The Fold Compound
  *  @{
- *
- *  @brief  This module provides interfaces that deal with the most basic data structure used
- *          in structure predicting and energy evaluating function of the RNAlib.
- *
- *          Throughout the entire RNAlib, the #vrna_fold_compound_t, is used to group
- *          information and data that is required for structure prediction and energy evaluation.
- *          Here, you'll find interface functions to create, modify, and delete #vrna_fold_compound_t
- *          data structures.
  */
 
 /**
@@ -43,7 +47,31 @@ typedef struct vrna_fc_s vrna_fold_compound_t;
  *
  *  @param data    The data that needs to be free'd
  */
-typedef void (vrna_callback_free_auxdata)(void *data);
+typedef void (*vrna_auxdata_free_f)(void *data);
+
+
+/**
+ *  @brief  Callback to prepare user-provided data based on some event
+ *
+ *  @param  fc          The fold compound the event was emitted from
+ *  @param  data        The user-defined data that may be prepared according to the event
+ *  @param  event       The event
+ *  @param  event_data  Some additional data corresponding to the event
+ *  @return             non-zero if the preparation was successful, 0 otherwise
+ */
+typedef int (*vrna_auxdata_prepare_f)(vrna_fold_compound_t  *fc,
+                                      void                  *data,
+                                      unsigned int          event,
+                                      void                  *event_data);
+
+
+/**
+ *  @brief Callback to free memory allocated for auxiliary user-provided data
+ *
+ *  @deprecated   Use vrna_auxdata_free_f(void *data) instead!
+ */
+DEPRECATED(typedef void (vrna_callback_free_auxdata)(void *data),
+           "Use vrna_auxdata_free_f instead!");
 
 /**
  *  @brief Callback to perform specific user-defined actions before, or after recursive computations
@@ -60,17 +88,23 @@ typedef void (vrna_callback_free_auxdata)(void *data);
  *       vrna_pf(),
  *       #VRNA_STATUS_MFE_PRE, #VRNA_STATUS_MFE_POST, #VRNA_STATUS_PF_PRE, #VRNA_STATUS_PF_POST
  *
+ *  @param fc       The fold compound emitting the status
  *  @param status   The status indicator
  *  @param data     The data structure that was assigned with vrna_fold_compound_add_auxdata()
- *  @param status   The status indicator
  */
-typedef void (vrna_callback_recursion_status)(unsigned char status,
-                                              void          *data);
+typedef void (*vrna_recursion_status_f)(vrna_fold_compound_t *fc,
+                                        unsigned char status,
+                                        void          *data);
+
+DEPRECATED(typedef void (vrna_callback_recursion_status)(unsigned char status,
+                                              void          *data),
+           "Use vrna_recursion_status_f instead!");
+
 
 /**
  *  @brief  Status message indicating that MFE computations are about to begin
  *
- *  @see  #vrna_fold_compound_t.stat_cb, vrna_callback_recursion_status(), vrna_mfe(), vrna_fold(), vrna_circfold(),
+ *  @see  #vrna_fold_compound_t.stat_cb, vrna_recursion_status_f(), vrna_mfe(), vrna_fold(), vrna_circfold(),
  *        vrna_alifold(), vrna_circalifold(), vrna_cofold()
  */
 #define VRNA_STATUS_MFE_PRE     (unsigned char)1
@@ -78,7 +112,7 @@ typedef void (vrna_callback_recursion_status)(unsigned char status,
 /**
  *  @brief  Status message indicating that MFE computations are finished
  *
- *  @see  #vrna_fold_compound_t.stat_cb, vrna_callback_recursion_status(), vrna_mfe(), vrna_fold(), vrna_circfold(),
+ *  @see  #vrna_fold_compound_t.stat_cb, vrna_recursion_status_f(), vrna_mfe(), vrna_fold(), vrna_circfold(),
  *        vrna_alifold(), vrna_circalifold(), vrna_cofold()
  */
 #define VRNA_STATUS_MFE_POST    (unsigned char)2
@@ -86,27 +120,32 @@ typedef void (vrna_callback_recursion_status)(unsigned char status,
 /**
  *  @brief  Status message indicating that Partition function computations are about to begin
  *
- *  @see  #vrna_fold_compound_t.stat_cb, vrna_callback_recursion_status(), vrna_pf()
+ *  @see  #vrna_fold_compound_t.stat_cb, vrna_recursion_status_f(), vrna_pf()
  */
 #define VRNA_STATUS_PF_PRE      (unsigned char)3
 
 /**
  *  @brief  Status message indicating that Partition function computations are finished
  *
- *  @see  #vrna_fold_compound_t.stat_cb, vrna_callback_recursion_status(), vrna_pf()
+ *  @see  #vrna_fold_compound_t.stat_cb, vrna_recursion_status_f(), vrna_pf()
  */
 #define VRNA_STATUS_PF_POST     (unsigned char)4
 
 
 #include <ViennaRNA/model.h>
 #include <ViennaRNA/params/basic.h>
-#include <ViennaRNA/sequence.h>
-#include <ViennaRNA/dp_matrices.h>
+#include <ViennaRNA/sequences/sequence.h>
+#include <ViennaRNA/datastructures/dp_matrices.h>
 #include <ViennaRNA/constraints/hard.h>
 #include <ViennaRNA/constraints/soft.h>
-#include <ViennaRNA/grammar.h>
-#include "ViennaRNA/structured_domains.h"
-#include "ViennaRNA/unstructured_domains.h"
+#include <ViennaRNA/grammar/basic.h>
+#include <ViennaRNA/structured_domains.h>
+#include <ViennaRNA/unstructured_domains.h>
+
+#ifdef VRNA_WITH_SVM
+#include <ViennaRNA/zscore/basic.h>
+#endif
+
 
 /**
  *  @brief  An enumerator that is used to specify the type of a #vrna_fold_compound_t
@@ -121,10 +160,10 @@ typedef enum {
  *  @brief  The most basic data structure required by many functions throughout the RNAlib
  *
  *  @note   Please read the documentation of this data structure carefully! Some attributes are only available for
- *  specific types this data structure can adopt.
+ *          specific types this data structure can adopt.
  *
  *  @warning  Reading/Writing from/to attributes that are not within the scope of the current type usually result
- *  in undefined behavior!
+ *            in undefined behavior!
  *
  *  @see  #vrna_fold_compound_t.type, vrna_fold_compound(), vrna_fold_compound_comparative(), vrna_fold_compound_free(),
  *        #VRNA_FC_TYPE_SINGLE, #VRNA_FC_TYPE_COMPARATIVE
@@ -142,18 +181,22 @@ struct vrna_fc_s {
                                      *      attributes to use within this data structure.
                                      */
   unsigned int      length;         /**<  @brief  The length of the sequence (or sequence alignment) */
-  int               cutpoint;       /**<  @brief  The position of the (cofold) cutpoint within the provided sequence.
+#ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
+  DEPRECATED(int cutpoint,
+             "Use strand_* members instead");
+                                    /**<  @brief  The position of the (cofold) cutpoint within the provided sequence.
                                      * If there is no cutpoint, this field will be set to -1
                                      */
-
+#endif
   unsigned int      *strand_number; /**<  @brief  The strand number a particular nucleotide is associated with */
   unsigned int      *strand_order;  /**<  @brief  The strand order, i.e. permutation of current concatenated sequence */
+  unsigned int      *strand_order_uniq; /**<  @brief  The strand order array where identical sequences have the same ID */
   unsigned int      *strand_start;  /**<  @brief  The start position of a particular strand within the current concatenated sequence */
   unsigned int      *strand_end;    /**<  @brief  The end (last) position of a particular strand within the current concatenated sequence */
 
-  unsigned int      strands;
-  vrna_seq_t        *nucleotides;
-  vrna_msa_t        *alignment;
+  unsigned int      strands;        /**<  @brief  Number of interacting strands */
+  vrna_seq_t        *nucleotides;   /**<  @brief  Set of nucleotide sequences */
+  vrna_msa_t        *alignment;     /**<  @brief  Set of alignments */
 
   vrna_hc_t         *hc;            /**<  @brief  The hard constraints data structure used for structure prediction */
 
@@ -172,18 +215,18 @@ struct vrna_fc_s {
    *  @name User-defined data fields
    *  @{
    */
-  vrna_callback_recursion_status  *stat_cb;       /**<  @brief  Recursion status callback (usually called just before, and
-                                                   *            after recursive computations in the library
-                                                   *    @see    vrna_callback_recursion_status(), vrna_fold_compound_add_callback()
-                                                   */
+  vrna_recursion_status_f   stat_cb;       /**<  @brief  Recursion status callback (usually called just before, and
+                                            *            after recursive computations in the library
+                                            *    @see    vrna_recursion_status_f(), vrna_fold_compound_add_callback()
+                                            */
 
-  void                            *auxdata;       /**<  @brief  A pointer to auxiliary, user-defined data
-                                                   *    @see vrna_fold_compound_add_auxdata(), #vrna_fold_compound_t.free_auxdata
-                                                   */
+  void                      *auxdata;      /**<  @brief  A pointer to auxiliary, user-defined data
+                                            *    @see vrna_fold_compound_add_auxdata(), #vrna_fold_compound_t.free_auxdata
+                                            */
 
-  vrna_callback_free_auxdata      *free_auxdata;  /**<  @brief A callback to free auxiliary user data whenever the fold_compound itself is free'd
-                                                   *    @see  #vrna_fold_compound_t.auxdata, vrna_callback_free_auxdata()
-                                                   */
+  vrna_auxdata_free_f       free_auxdata;  /**<  @brief A callback to free auxiliary user data whenever the fold_compound itself is free'd
+                                            *    @see  #vrna_fold_compound_t.auxdata, vrna_auxdata_free_f()
+                                            */
 
   /**
    *  @}
@@ -199,7 +242,7 @@ struct vrna_fc_s {
   vrna_ud_t     *domains_up;                /**<  @brief  Additional unstructured domains */
 
   /* auxiliary (user-defined) extension to the folding grammar */
-  vrna_gr_aux_t *aux_grammar;
+  vrna_gr_aux_t aux_grammar;               /**<  @brief  Additional decomposition grammar rules */
 
   /**
    *  @}
@@ -222,7 +265,12 @@ struct vrna_fc_s {
                                          *    @see    vrna_sequence_encode()
                                          *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
                                          */
-  short *sequence_encoding2;
+      short *encoding5;
+      short *encoding3;
+
+      short *sequence_encoding2;
+
+
       char *ptype;                      /**<  @brief  Pair type array
                                          *
                                          *    Contains the numerical encoding of the pair type for each pair (i,j) used
@@ -332,6 +380,10 @@ struct {
    */
   int   window_size;              /**<  @brief  window size for local folding sliding window approach */
   char  **ptype_local;            /**<  @brief  Pair type array (for local folding) */
+#ifdef VRNA_WITH_SVM
+  vrna_zsc_dat_t  zscore_data;    /**<  @brief  Data structure with settings for z-score computations */
+#endif
+
   /**
    *  @}
    */
@@ -351,7 +403,7 @@ struct {
  *
  *  @see vrna_fold_compound(), vrna_fold_compound_comparative(), #VRNA_OPTION_EVAL_ONLY
  */
-#define VRNA_OPTION_MFE             1U
+#define VRNA_OPTION_MFE             (1 << 0)
 
 /**
  *  @brief  Option flag to specify requirement of Partition Function (PF) DP matrices
@@ -359,12 +411,12 @@ struct {
  *
  *  @see vrna_fold_compound(), vrna_fold_compound_comparative(), #VRNA_OPTION_EVAL_ONLY
  */
-#define VRNA_OPTION_PF              2U
+#define VRNA_OPTION_PF              (1 << 1)
 
 /**
  *  @brief  Option flag to specify requirement of dimer DP matrices
  */
-#define VRNA_OPTION_HYBRID          4U
+#define VRNA_OPTION_HYBRID          (1 << 2)
 
 /**
  *  @brief  Option flag to specify that neither MFE, nor PF DP matrices are required
@@ -375,12 +427,18 @@ struct {
  *
  *  @see vrna_fold_compound(), vrna_fold_compound_comparative(), vrna_eval_structure()
  */
-#define VRNA_OPTION_EVAL_ONLY       8U
+#define VRNA_OPTION_EVAL_ONLY       (1 << 3)
 
 /**
  *  @brief  Option flag to specify requirement of DP matrices for local folding approaches
  */
-#define VRNA_OPTION_WINDOW          16U
+#define VRNA_OPTION_WINDOW          (1 << 4)
+
+
+#define VRNA_OPTION_F5              (1 << 5)
+#define VRNA_OPTION_F3              (1 << 6)
+#define VRNA_OPTION_WINDOW_F5       (VRNA_OPTION_WINDOW | VRNA_OPTION_F5)
+#define VRNA_OPTION_WINDOW_F3       (VRNA_OPTION_WINDOW | VRNA_OPTION_F3)
 
 /**
  *  @brief  Retrieve a #vrna_fold_compound_t data structure for single sequences and hybridizing sequences
@@ -420,9 +478,9 @@ struct {
  *  @return               A prefilled vrna_fold_compound_t ready to be used for computations (may be @p NULL on error)
  */
 vrna_fold_compound_t *
-vrna_fold_compound(const char   *sequence,
-                   vrna_md_t    *md_p,
-                   unsigned int options);
+vrna_fold_compound(const char       *sequence,
+                   const vrna_md_t  *md_p,
+                   unsigned int     options);
 
 
 /**
@@ -506,23 +564,23 @@ vrna_fold_compound_free(vrna_fold_compound_t *fc);
  *  @brief  Add auxiliary data to the #vrna_fold_compound_t
  *
  *  This function allows one to bind arbitrary data to a #vrna_fold_compound_t which may later on be used
- *  by one of the callback functions, e.g. vrna_callback_recursion_status(). To allow for proper cleanup
+ *  by one of the callback functions, e.g. vrna_recursion_status_f(). To allow for proper cleanup
  *  of the memory occupied by this auxiliary data, the user may also provide a pointer to a cleanup function
  *  that free's the corresponding memory. This function will be called automatically when the #vrna_fold_compound_t
  *  is free'd with vrna_fold_compound_free().
  *
- *  @note Before attaching the arbitrary data pointer, this function will call the vrna_callback_free_auxdata()
+ *  @note Before attaching the arbitrary data pointer, this function will call the vrna_auxdata_free_f()
  *        on any pre-existing data that is already attached.
  *
- *  @see vrna_callback_free_auxdata()
+ *  @see vrna_auxdata_free_f()
  *  @param  fc    The fold_compound the arbitrary data pointer should be associated with
  *  @param  data  A pointer to an arbitrary data structure
  *  @param  f     A pointer to function that free's memory occupied by the arbitrary data (May be NULL)
  */
 void
-vrna_fold_compound_add_auxdata(vrna_fold_compound_t       *fc,
-                               void                       *data,
-                               vrna_callback_free_auxdata *f);
+vrna_fold_compound_add_auxdata(vrna_fold_compound_t *fc,
+                               void                 *data,
+                               vrna_auxdata_free_f  f);
 
 
 /**
@@ -534,15 +592,15 @@ vrna_fold_compound_add_auxdata(vrna_fold_compound_t       *fc,
  *  #vrna_fold_compound_t, and a status message. Hence, it has complete access to all variables that
  *  incluence the recursive computations.
  *
- *  @see  vrna_callback_recursion_status(), #vrna_fold_compound_t,
+ *  @see  vrna_recursion_status_f(), #vrna_fold_compound_t,
  *        #VRNA_STATUS_MFE_PRE, #VRNA_STATUS_MFE_POST, #VRNA_STATUS_PF_PRE, #VRNA_STATUS_PF_POST
  *
  *  @param  fc    The fold_compound the callback function should be attached to
  *  @param  f     The pointer to the recursion status callback function
  */
 void
-vrna_fold_compound_add_callback(vrna_fold_compound_t            *fc,
-                                vrna_callback_recursion_status  *f);
+vrna_fold_compound_add_callback(vrna_fold_compound_t    *fc,
+                                vrna_recursion_status_f f);
 
 
 /**
