@@ -16,6 +16,8 @@
 #include <math.h>
 #include <stdarg.h>
 #include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/log.h"
+#include "ViennaRNA/model.h"
 #include "ViennaRNA/io/utils.h"
 #include "ViennaRNA/params/constants.h"
 #include "ViennaRNA/params/default.h"
@@ -202,7 +204,7 @@ rd_6dim(char    **content,
         size_t  *line_no,
         int     *array,
         int     dim[6],
-        int     shift[6]);
+        int     shift[6]) VRNA_UNUSED;
 
 
 PRIVATE void
@@ -250,7 +252,7 @@ save_parameter_file(const char    fname[],
  */
 PUBLIC int
 vrna_params_load(const char   fname[],
-                 unsigned int options)
+                 unsigned int options VRNA_UNUSED)
 {
   char  *name, **file_content, **ptr;
   int   ret;
@@ -286,7 +288,7 @@ vrna_params_save(const char   fname[],
 PUBLIC int
 vrna_params_load_from_string(const char   *string,
                              const char   *name,
-                             unsigned int options)
+                             unsigned int options VRNA_UNUSED)
 {
   int ret = 0;
 
@@ -300,9 +302,16 @@ vrna_params_load_from_string(const char   *string,
     /* convert string into array of lines */
     tmp_string = strdup(string);
 
-    token = strtok_r(tmp_string, "\n", &rest);
+    token = tmp_string;
+
+    /* tokenize the input string by separating lines at '\n' */
+    while (1) {
+      rest = strchr(token, '\n');
+      if (rest != NULL)
+        *rest = '\0';
+      else
+        break;
 
-    while (token != NULL) {
       if (lines == lines_mem) {
         lines_mem     += 32768;
         params_array  = (char **)vrna_realloc(params_array, sizeof(char *) * lines_mem);
@@ -310,7 +319,7 @@ vrna_params_load_from_string(const char   *string,
 
       params_array[lines++] = strdup(token);
 
-      token = strtok_r(NULL, "\n", &rest);
+      token = rest + 1;
     }
 
     /* reallocate to actual requirements */
@@ -344,7 +353,10 @@ vrna_params_load_defaults(void)
 PUBLIC int
 vrna_params_load_RNA_Turner2004(void)
 {
-  return vrna_params_load_from_string(parameter_set_rna_turner2004,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_RNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_RNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_RNA);
+  return vrna_params_load_from_string((const char *)parameter_set_rna_turner2004,
                                       "RNA - Turner 2004",
                                       0);
 }
@@ -353,7 +365,10 @@ vrna_params_load_RNA_Turner2004(void)
 PUBLIC int
 vrna_params_load_RNA_Turner1999(void)
 {
-  return vrna_params_load_from_string(parameter_set_rna_turner1999,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_RNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_RNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_RNA);
+  return vrna_params_load_from_string((const char *)parameter_set_rna_turner1999,
                                       "RNA - Turner 1999",
                                       0);
 }
@@ -362,7 +377,10 @@ vrna_params_load_RNA_Turner1999(void)
 PUBLIC int
 vrna_params_load_RNA_Andronescu2007(void)
 {
-  return vrna_params_load_from_string(parameter_set_rna_andronescu2007,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_RNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_RNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_RNA);
+  return vrna_params_load_from_string((const char *)parameter_set_rna_andronescu2007,
                                       "RNA - Andronescu 2007",
                                       0);
 }
@@ -371,7 +389,10 @@ vrna_params_load_RNA_Andronescu2007(void)
 PUBLIC int
 vrna_params_load_RNA_Langdon2018(void)
 {
-  return vrna_params_load_from_string(parameter_set_rna_langdon2018,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_RNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_RNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_RNA);
+  return vrna_params_load_from_string((const char *)parameter_set_rna_langdon2018,
                                       "RNA - Langdon 2018",
                                       0);
 }
@@ -380,7 +401,10 @@ vrna_params_load_RNA_Langdon2018(void)
 PUBLIC int
 vrna_params_load_RNA_misc_special_hairpins(void)
 {
-  return vrna_params_load_from_string(parameter_set_rna_misc_special_hairpins,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_RNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_RNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_RNA);
+  return vrna_params_load_from_string((const char *)parameter_set_rna_misc_special_hairpins,
                                       "RNA - Misc. Special Hairpins",
                                       0);
 }
@@ -389,7 +413,10 @@ vrna_params_load_RNA_misc_special_hairpins(void)
 PUBLIC int
 vrna_params_load_DNA_Mathews2004(void)
 {
-  return vrna_params_load_from_string(parameter_set_dna_mathews2004,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_DNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_DNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_DNA);
+  return vrna_params_load_from_string((const char *)parameter_set_dna_mathews2004,
                                       "DNA - Mathews 2004",
                                       0);
 }
@@ -398,7 +425,10 @@ vrna_params_load_DNA_Mathews2004(void)
 PUBLIC int
 vrna_params_load_DNA_Mathews1999(void)
 {
-  return vrna_params_load_from_string(parameter_set_dna_mathews1999,
+  vrna_md_defaults_helical_rise(VRNA_MODEL_HELICAL_RISE_DNA);
+  vrna_md_defaults_backbone_length(VRNA_MODEL_BACKBONE_LENGTH_DNA);
+  vrna_md_defaults_saltDPXInitFact(VRNA_MODEL_SALT_DPXINIT_FACT_DNA);
+  return vrna_params_load_from_string((const char *)parameter_set_dna_mathews1999,
                                       "DNA - Mathews 1999",
                                       0);
 }
@@ -441,7 +471,7 @@ file2array(const char fname[])
 
     fclose(fp);
   } else {
-    vrna_message_warning("read_parameter_file():"
+    vrna_log_warning("read_parameter_file():"
                          "Can't open file %s\n",
                          fname);
   }
@@ -470,7 +500,7 @@ set_parameters_from_string(char       **file_content,
   last_param_file = (name) ? strdup(name) : NULL;
 
   if (strncmp(file_content[line_no++], "## RNAfold parameter file v2.0", 30) != 0) {
-    vrna_message_warning("Missing header line in file.\n"
+    vrna_log_warning("Missing header line in file.\n"
                          "May be this file has not v2.0 format.\n"
                          "Use INTERRUPT-key to stop.");
   }
@@ -661,7 +691,7 @@ set_parameters_from_string(char       **file_content,
           rd_Hexaloop37(file_content, &line_no);
           break;
         default:      /* do nothing but complain */
-          vrna_message_warning("read_epars: Unknown field identifier in `%s'", line);
+          vrna_log_warning("read_epars: Unknown field identifier in `%s'", line);
       }
     } /* else ignore line */
   }
@@ -721,8 +751,10 @@ get_array1(char   **content,
   i = last = 0;
   while (i < size) {
     line = content[(*line_no)++];
-    if (!line)
-      vrna_message_error("unexpected end of file in get_array1");
+    if (!line) {
+      vrna_log_error("unexpected end of file in get_array1");
+      return NULL;
+    }
 
     ignore_comment(line);
     pos = 0;
@@ -733,9 +765,10 @@ get_array1(char   **content,
         continue;
       } else if (buf[0] == 'x') {
         /* should only be used for loop parameters */
-        if (i == 0)
-          vrna_message_error("can't extrapolate first value");
-
+        if (i == 0) {
+          vrna_log_error("can't extrapolate first value");
+          return NULL;
+        }
         p = arr[last] + (int)(0.5 + lxc37 * log(((double)i) / (double)(last)));
       } else if (strcmp(buf, "DEF") == 0) {
         p = DEF;
@@ -746,9 +779,8 @@ get_array1(char   **content,
       } else {
         r = sscanf(buf, "%d", &p);
         if (r != 1) {
+          vrna_log_error("can't interpret `%s' in get_array1", buf);
           return line + pos;
-          vrna_message_error("can't interpret `%s' in get_array1", buf);
-          exit(1);
         }
 
         last = i;
@@ -785,10 +817,8 @@ rd_1dim_slice(char    **content,
 
   cp = get_array1(content, line_no, array + shift, dim - shift - post);
 
-  if (cp) {
-    vrna_message_error("\nrd_1dim: %s", cp);
-    exit(1);
-  }
+  if (cp)
+    vrna_log_error("\nrd_1dim: %s", cp);
 
   return;
 }
@@ -1036,6 +1066,10 @@ rd_Tetraloop37(char   **content,
     strcat(Tetraloops, " ");
     i++;
   } while ((r == 3) && (i < 40));
+
+  /* decrease line number to continue parsing at line that failed before */
+  (*line_no)--;
+
   return;
 }
 
@@ -1062,6 +1096,10 @@ rd_Hexaloop37(char    **content,
     strcat(Hexaloops, " ");
     i++;
   } while ((r == 3) && (i < 40));
+
+  /* decrease line number to continue parsing at line that failed before */
+  (*line_no)--;
+
   return;
 }
 
@@ -1088,6 +1126,10 @@ rd_Triloop37(char   **content,
     strcat(Triloops, " ");
     i++;
   } while ((r == 3) && (i < 40));
+
+  /* decrease line number to continue parsing at line that failed before */
+  (*line_no)--;
+
   return;
 }
 
@@ -1106,8 +1148,10 @@ ignore_comment(char *line)
 
   if ((cp1 = strstr(line, "/*"))) {
     cp2 = strstr(cp1, "*/");
-    if (cp2 == NULL)
-      vrna_message_error("unclosed comment in parameter file");
+    if (cp2 == NULL) {
+      vrna_log_error("unclosed comment in parameter file");
+      return;
+    }
 
     /* can't use strcpy for overlapping strings */
     for (cp2 += 2; *cp2 != '\0'; cp2++, cp1++)
@@ -1129,20 +1173,20 @@ check_symmetry(void)
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       if (stack37[i][j] != stack37[j][i])
-        vrna_message_warning("stacking energies not symmetric");
+        vrna_log_warning("stacking energies not symmetric");
 
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       if (stackdH[i][j] != stackdH[j][i])
-        vrna_message_warning("stacking enthalpies not symmetric");
+        vrna_log_warning("stacking enthalpies not symmetric");
 
-  /* interior 1x1 loops */
+  /* internal 1x1 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
         for (l = 0; l < 5; l++)
           if (int11_37[i][j][k][l] != int11_37[j][i][l][k])
-            vrna_message_warning("int11 energies not symmetric (%d,%d,%d,%d) (%d vs. %d)",
+            vrna_log_warning("int11 energies not symmetric (%d,%d,%d,%d) (%d vs. %d)",
                                  i, j, k, l, int11_37[i][j][k][l], int11_37[j][i][l][k]);
 
   for (i = 0; i <= NBPAIRS; i++)
@@ -1150,9 +1194,9 @@ check_symmetry(void)
       for (k = 0; k < 5; k++)
         for (l = 0; l < 5; l++)
           if (int11_dH[i][j][k][l] != int11_dH[j][i][l][k])
-            vrna_message_warning("int11 enthalpies not symmetric");
+            vrna_log_warning("int11 enthalpies not symmetric");
 
-  /* interior 2x2 loops */
+  /* internal 2x2 loops */
   for (i = 0; i <= NBPAIRS; i++)
     for (j = 0; j <= NBPAIRS; j++)
       for (k = 0; k < 5; k++)
@@ -1161,7 +1205,7 @@ check_symmetry(void)
           for (m = 0; m < 5; m++)
             for (n = 0; n < 5; n++)
               if (int22_37[i][j][k][l][m][n] != int22_37[j][i][m][n][k][l])
-                vrna_message_warning("int22 energies not symmetric");
+                vrna_log_warning("int22 energies not symmetric");
         }
 
   for (i = 0; i <= NBPAIRS; i++)
@@ -1172,7 +1216,7 @@ check_symmetry(void)
           for (m = 0; m < 5; m++)
             for (n = 0; n < 5; n++)
               if (int22_dH[i][j][k][l][m][n] != int22_dH[j][i][m][n][k][l])
-                vrna_message_warning("int22 enthalpies not symmetric: %d %d %d %d %d %d",
+                vrna_log_warning("int22 enthalpies not symmetric: %d %d %d %d %d %d",
                                      i, j, k, l, m, n);
         }
 }
@@ -1300,10 +1344,10 @@ update_nst(int array[NBPAIRS + 1][NBPAIRS + 1][5][5][5][5])
 
 PRIVATE int
 save_parameter_file(const char    fname[],
-                    unsigned int  options)
+                    unsigned int  options VRNA_UNUSED)
 {
   FILE  *outfp;
-  int   c;
+  size_t   c;
   char  *pnames[] = {
     "NP", "CG", "GC", "GU", "UG", "AU", "UA", " @"
   };
@@ -1311,7 +1355,7 @@ save_parameter_file(const char    fname[],
 
   outfp = fopen(fname, "w");
   if (!outfp) {
-    vrna_message_warning("can't open file %s", fname);
+    vrna_log_warning("can't open file %s", fname);
     return 0;
   }
 
@@ -1329,7 +1373,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMH));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchH37[k][i], 5, 5, outfp);
@@ -1337,7 +1381,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMH_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchHdH[k][i], 5, 5, outfp);
@@ -1345,7 +1389,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchI37[k][i], 5, 5, outfp);
@@ -1353,7 +1397,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchIdH[k][i], 5, 5, outfp);
@@ -1361,7 +1405,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI1N));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch1nI37[k][i], 5, 5, outfp);
@@ -1369,7 +1413,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI1N_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch1nIdH[k][i], 5, 5, outfp);
@@ -1377,7 +1421,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI23));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch23I37[k][i], 5, 5, outfp);
@@ -1385,7 +1429,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMI23_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch23IdH[k][i], 5, 5, outfp);
@@ -1393,7 +1437,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMM));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchM37[k][i], 5, 5, outfp);
@@ -1401,7 +1445,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MMM_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchMdH[k][i], 5, 5, outfp);
@@ -1409,7 +1453,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MME));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchExt37[k][i], 5, 5, outfp);
@@ -1417,7 +1461,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(MME_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchExtdH[k][i], 5, 5, outfp);
@@ -1444,10 +1488,10 @@ save_parameter_file(const char    fname[],
     display_array(dangle3_dH[c], 5, 5, outfp);
 
 
-  /* dont print "no pair" entries for interior loop arrays */
+  /* dont print "no pair" entries for internal loop arrays */
   fprintf(outfp, "\n# %s\n", settype(INT11));
   {
-    int i, k, l;
+    size_t i, k, l;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (l = 1; l < NBPAIRS + 1; l++) {
         fprintf(outfp, "/* %2s..%2s */\n", pnames[k], pnames[l]);
@@ -1458,7 +1502,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(INT11_H));
   {
-    int i, k, l;
+    size_t i, k, l;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (l = 1; l < NBPAIRS + 1; l++) {
         fprintf(outfp, "/* %2s..%2s */\n", pnames[k], pnames[l]);
@@ -1469,7 +1513,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(INT21));
   {
-    int p1, p2, i, j;
+    size_t p1, p2, i, j;
     for (p1 = 1; p1 < NBPAIRS + 1; p1++)
       for (p2 = 1; p2 < NBPAIRS + 1; p2++)
         for (i = 0; i < 5; i++) {
@@ -1482,7 +1526,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(INT21_H));
   {
-    int p1, p2, i, j;
+    size_t p1, p2, i, j;
     for (p1 = 1; p1 < NBPAIRS + 1; p1++)
       for (p2 = 1; p2 < NBPAIRS + 1; p2++)
         for (i = 0; i < 5; i++) {
@@ -1495,7 +1539,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(INT22));
   {
-    int p1, p2, i, j, k;
+    size_t p1, p2, i, j, k;
     for (p1 = 1; p1 < NBPAIRS; p1++)
       for (p2 = 1; p2 < NBPAIRS; p2++)
         for (i = 1; i < 5; i++)
@@ -1509,7 +1553,7 @@ save_parameter_file(const char    fname[],
 
   fprintf(outfp, "\n# %s\n", settype(INT22_H));
   {
-    int p1, p2, i, j, k;
+    size_t p1, p2, i, j, k;
     for (p1 = 1; p1 < NBPAIRS; p1++)
       for (p2 = 1; p2 < NBPAIRS; p2++)
         for (i = 1; i < 5; i++)
@@ -1605,7 +1649,9 @@ last_parameter_file(void)
 PUBLIC void
 read_parameter_file(const char fname[])
 {
-  (void)vrna_params_load(fname, VRNA_PARAMETER_FORMAT_DEFAULT);
+  int ret = vrna_params_load(fname, VRNA_PARAMETER_FORMAT_DEFAULT);
+  if (!ret)
+    vrna_log_warning("Failed to load parameters from file \"%s\"", fname);
 }
 
 
@@ -1626,9 +1672,9 @@ settype(enum parset s)
     case      B_H:
       return "bulge_enthalpies";
     case       IL:
-      return "interior";
+      return "internal";
     case     IL_H:
-      return "interior_enthalpies";
+      return "internal_enthalpies";
     case      MME:
       return "mismatch_exterior";
     case    MME_H:
@@ -1638,17 +1684,17 @@ settype(enum parset s)
     case    MMH_H:
       return "mismatch_hairpin_enthalpies";
     case      MMI:
-      return "mismatch_interior";
+      return "mismatch_internal";
     case    MMI_H:
-      return "mismatch_interior_enthalpies";
+      return "mismatch_internal_enthalpies";
     case    MMI1N:
-      return "mismatch_interior_1n";
+      return "mismatch_internal_1n";
     case  MMI1N_H:
-      return "mismatch_interior_1n_enthalpies";
+      return "mismatch_internal_1n_enthalpies";
     case    MMI23:
-      return "mismatch_interior_23";
+      return "mismatch_internal_23";
     case  MMI23_H:
-      return "mismatch_interior_23_enthalpies";
+      return "mismatch_internal_23_enthalpies";
     case      MMM:
       return "mismatch_multi";
     case    MMM_H:
@@ -1688,7 +1734,7 @@ settype(enum parset s)
     case     MISC:
       return "Misc";
     default:
-      vrna_message_error("\nThe answer is: 42\n");
+      vrna_log_error("\nThe answer is: 42\n");
   }
   return "";
 }
@@ -1699,82 +1745,107 @@ settype(enum parset s)
 PUBLIC enum parset
 gettype(const char *ident)
 {
-  if (strcmp(ident, "stack") == 0)
+  if (strcmp(ident, "stack") == 0) {
     return S;
-  else if (strcmp(ident, "stack_enthalpies") == 0)
+  } else if (strcmp(ident, "stack_enthalpies") == 0) {
     return S_H;
-  else if (strcmp(ident, "hairpin") == 0)
+  } else if (strcmp(ident, "hairpin") == 0) {
     return HP;
-  else if (strcmp(ident, "hairpin_enthalpies") == 0)
+  } else if (strcmp(ident, "hairpin_enthalpies") == 0) {
     return HP_H;
-  else if (strcmp(ident, "bulge") == 0)
+  } else if (strcmp(ident, "bulge") == 0) {
     return B;
-  else if (strcmp(ident, "bulge_enthalpies") == 0)
+  } else if (strcmp(ident, "bulge_enthalpies") == 0) {
     return B_H;
-  else if (strcmp(ident, "interior") == 0)
+  } else if (strcmp(ident, "interior") == 0) {
+    vrna_log_warning("Detected deprecated identifier 'interior'! Use 'internal' instead!");
     return IL;
-  else if (strcmp(ident, "interior_enthalpies") == 0)
+  } else if (strcmp(ident, "interior_enthalpies") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'interior_enthalpies'! Use 'internal_enthalpies' instead!");
     return IL_H;
-  else if (strcmp(ident, "mismatch_exterior") == 0)
+  } else if (strcmp(ident, "internal") == 0) {
+    return IL;
+  } else if (strcmp(ident, "internal_enthalpies") == 0) {
+    return IL_H;
+  } else if (strcmp(ident, "mismatch_exterior") == 0) {
     return MME;
-  else if (strcmp(ident, "mismatch_exterior_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_exterior_enthalpies") == 0) {
     return MME_H;
-  else if (strcmp(ident, "mismatch_hairpin") == 0)
+  } else if (strcmp(ident, "mismatch_hairpin") == 0) {
     return MMH;
-  else if (strcmp(ident, "mismatch_hairpin_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_hairpin_enthalpies") == 0) {
     return MMH_H;
-  else if (strcmp(ident, "mismatch_interior") == 0)
+  } else if (strcmp(ident, "mismatch_interior") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior'! Use 'mismatch_internal' instead!");
     return MMI;
-  else if (strcmp(ident, "mismatch_interior_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_interior_enthalpies") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior_enthalpies'! Use 'mismatch_internal_enthalpies' instead!");
     return MMI_H;
-  else if (strcmp(ident, "mismatch_interior_1n") == 0)
+  } else if (strcmp(ident, "mismatch_interior_1n") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior_1n'! Use 'mismatch_internal_1n' instead!");
     return MMI1N;
-  else if (strcmp(ident, "mismatch_interior_1n_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_interior_1n_enthalpies") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior_1n_enthalpies'! Use 'mismatch_internal_1n_enthalpies' instead!");
     return MMI1N_H;
-  else if (strcmp(ident, "mismatch_interior_23") == 0)
+  } else if (strcmp(ident, "mismatch_interior_23") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior_23'! Use 'mismatch_internal_23' instead!");
     return MMI23;
-  else if (strcmp(ident, "mismatch_interior_23_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_interior_23_enthalpies") == 0) {
+    vrna_log_warning("Detected  deprecated identifier 'mismatch_interior_23_enthalpies'! Use 'mismatch_internal_23_enthalpies' instead!");
     return MMI23_H;
-  else if (strcmp(ident, "mismatch_multi") == 0)
+  } else if (strcmp(ident, "mismatch_internal") == 0) {
+    return MMI;
+  } else if (strcmp(ident, "mismatch_internal_enthalpies") == 0) {
+    return MMI_H;
+  } else if (strcmp(ident, "mismatch_internal_1n") == 0) {
+    return MMI1N;
+  } else if (strcmp(ident, "mismatch_internal_1n_enthalpies") == 0) {
+    return MMI1N_H;
+  } else if (strcmp(ident, "mismatch_internal_23") == 0) {
+    return MMI23;
+  } else if (strcmp(ident, "mismatch_internal_23_enthalpies") == 0) {
+    return MMI23_H;
+  } else if (strcmp(ident, "mismatch_multi") == 0) {
     return MMM;
-  else if (strcmp(ident, "mismatch_multi_enthalpies") == 0)
+  } else if (strcmp(ident, "mismatch_multi_enthalpies") == 0) {
     return MMM_H;
-  else if (strcmp(ident, "int11") == 0)
+  } else if (strcmp(ident, "int11") == 0) {
     return INT11;
-  else if (strcmp(ident, "int11_enthalpies") == 0)
+  } else if (strcmp(ident, "int11_enthalpies") == 0) {
     return INT11_H;
-  else if (strcmp(ident, "int21") == 0)
+  } else if (strcmp(ident, "int21") == 0) {
     return INT21;
-  else if (strcmp(ident, "int21_enthalpies") == 0)
+  } else if (strcmp(ident, "int21_enthalpies") == 0) {
     return INT21_H;
-  else if (strcmp(ident, "int22") == 0)
+  } else if (strcmp(ident, "int22") == 0) {
     return INT22;
-  else if (strcmp(ident, "int22_enthalpies") == 0)
+  } else if (strcmp(ident, "int22_enthalpies") == 0) {
     return INT22_H;
-  else if (strcmp(ident, "dangle5") == 0)
+  } else if (strcmp(ident, "dangle5") == 0) {
     return D5;
-  else if (strcmp(ident, "dangle5_enthalpies") == 0)
+  } else if (strcmp(ident, "dangle5_enthalpies") == 0) {
     return D5_H;
-  else if (strcmp(ident, "dangle3") == 0)
+  } else if (strcmp(ident, "dangle3") == 0) {
     return D3;
-  else if (strcmp(ident, "dangle3_enthalpies") == 0)
+  } else if (strcmp(ident, "dangle3_enthalpies") == 0) {
     return D3_H;
-  else if (strcmp(ident, "ML_params") == 0)
+  } else if (strcmp(ident, "ML_params") == 0) {
     return ML;
-  else if (strcmp(ident, "NINIO") == 0)
+  } else if (strcmp(ident, "NINIO") == 0) {
     return NIN;
-  else if (strcmp(ident, "Triloops") == 0)
+  } else if (strcmp(ident, "Triloops") == 0) {
     return TRI;
-  else if (strcmp(ident, "Tetraloops") == 0)
+  } else if (strcmp(ident, "Tetraloops") == 0) {
     return TL;
-  else if (strcmp(ident, "Hexaloops") == 0)
+  } else if (strcmp(ident, "Hexaloops") == 0) {
     return HEX;
-  else if (strcmp(ident, "Misc") == 0)
+  } else if (strcmp(ident, "Misc") == 0) {
     return MISC;
-  else if (strcmp(ident, "END") == 0)
+  } else if (strcmp(ident, "END") == 0) {
     return QUIT;
-  else
+  } else {
     return UNKNOWN;
+  }
 }
 
 
@@ -1784,7 +1855,7 @@ PUBLIC void
 write_parameter_file(const char fname[])
 {
   FILE  *outfp;
-  int   c;
+  size_t   c;
   char  *pnames[] = {
     "NP", "CG", "GC", "GU", "UG", "AU", "UA", " @"
   };
@@ -1792,8 +1863,8 @@ write_parameter_file(const char fname[])
 
   outfp = fopen(fname, "w");
   if (!outfp) {
-    vrna_message_error("can't open file %s", fname);
-    exit(1);
+    vrna_log_error("can't open file %s", fname);
+    return;
   }
 
   fprintf(outfp, "## RNAfold parameter file v2.0\n");
@@ -1810,7 +1881,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMH));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchH37[k][i], 5, 5, outfp);
@@ -1818,7 +1889,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMH_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchHdH[k][i], 5, 5, outfp);
@@ -1826,7 +1897,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchI37[k][i], 5, 5, outfp);
@@ -1834,7 +1905,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchIdH[k][i], 5, 5, outfp);
@@ -1842,7 +1913,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI1N));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch1nI37[k][i], 5, 5, outfp);
@@ -1850,7 +1921,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI1N_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch1nIdH[k][i], 5, 5, outfp);
@@ -1858,7 +1929,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI23));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch23I37[k][i], 5, 5, outfp);
@@ -1866,7 +1937,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMI23_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatch23IdH[k][i], 5, 5, outfp);
@@ -1874,7 +1945,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMM));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchM37[k][i], 5, 5, outfp);
@@ -1882,7 +1953,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MMM_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchMdH[k][i], 5, 5, outfp);
@@ -1890,7 +1961,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MME));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchExt37[k][i], 5, 5, outfp);
@@ -1898,7 +1969,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(MME_H));
   {
-    int i, k;
+    size_t i, k;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (i = 0; i < 5; i++)
         display_array(mismatchExtdH[k][i], 5, 5, outfp);
@@ -1925,10 +1996,10 @@ write_parameter_file(const char fname[])
     display_array(dangle3_dH[c], 5, 5, outfp);
 
 
-  /* dont print "no pair" entries for interior loop arrays */
+  /* dont print "no pair" entries for internal loop arrays */
   fprintf(outfp, "\n# %s\n", settype(INT11));
   {
-    int i, k, l;
+    size_t i, k, l;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (l = 1; l < NBPAIRS + 1; l++) {
         fprintf(outfp, "/* %2s..%2s */\n", pnames[k], pnames[l]);
@@ -1939,7 +2010,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(INT11_H));
   {
-    int i, k, l;
+    size_t i, k, l;
     for (k = 1; k < NBPAIRS + 1; k++)
       for (l = 1; l < NBPAIRS + 1; l++) {
         fprintf(outfp, "/* %2s..%2s */\n", pnames[k], pnames[l]);
@@ -1950,7 +2021,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(INT21));
   {
-    int p1, p2, i, j;
+    size_t p1, p2, i, j;
     for (p1 = 1; p1 < NBPAIRS + 1; p1++)
       for (p2 = 1; p2 < NBPAIRS + 1; p2++)
         for (i = 0; i < 5; i++) {
@@ -1963,7 +2034,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(INT21_H));
   {
-    int p1, p2, i, j;
+    size_t p1, p2, i, j;
     for (p1 = 1; p1 < NBPAIRS + 1; p1++)
       for (p2 = 1; p2 < NBPAIRS + 1; p2++)
         for (i = 0; i < 5; i++) {
@@ -1976,7 +2047,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(INT22));
   {
-    int p1, p2, i, j, k;
+    size_t p1, p2, i, j, k;
     for (p1 = 1; p1 < NBPAIRS; p1++)
       for (p2 = 1; p2 < NBPAIRS; p2++)
         for (i = 1; i < 5; i++)
@@ -1990,7 +2061,7 @@ write_parameter_file(const char fname[])
 
   fprintf(outfp, "\n# %s\n", settype(INT22_H));
   {
-    int p1, p2, i, j, k;
+    size_t p1, p2, i, j, k;
     for (p1 = 1; p1 < NBPAIRS; p1++)
       for (p2 = 1; p2 < NBPAIRS; p2++)
         for (i = 1; i < 5; i++)
