@@ -50,6 +50,14 @@
 #include "probing_data_helpers.h"
 
 
+#ifndef INLINE
+# ifdef __GNUC__
+#   define INLINE inline
+# else
+#   define INLINE
+# endif
+#endif
+
 struct options {
   int             filename_full;
   char            *filename_delim;
@@ -173,6 +181,10 @@ compute_MEA(vrna_fold_compound_t  *fc,
             vrna_cstr_t           rec_output);
 
 
+PRIVATE INLINE int
+get_stored_bp_contributions(vrna_sc_bp_storage_t  *container,
+                            unsigned int          j);
+
 /*--------------------------------------------------------------------------*/
 
 void
@@ -826,7 +838,7 @@ process_record(struct record_data *record)
   }
 
   if (opt->shape) {
-    vrna_constraints_add_SHAPE(vc,
+    constraints_add_SHAPE(vc,
                                opt->shape_file,
                                opt->shape_method,
                                opt->shape_conversion,
@@ -1063,8 +1075,90 @@ process_record(struct record_data *record)
                                                          vc->nucleotides[0].string);
 
       char                  *ss_AA  = (char *)vrna_alloc(sizeof(char) * (2 * Alength + 1));
+
+
       vrna_fold_compound_t  *fc_AA  = vrna_fold_compound(seq_AA, &(opt->md), VRNA_OPTION_DEFAULT);
 
+      /* extract hard constraints from concatenated input above, if any non-defaults */
+      if (vc->hc) {
+        vrna_smx_csr(vrna_uchar) *hc_nondefaults = vrna_hc_nondefaults(vc);
+
+        if (hc_nondefaults) {
+          /* single nucleotide constraints first */
+          for (unsigned int i = 1; i <= Alength; ++i) {
+            unsigned char constraint = vrna_smx_csr_get(hc_nondefaults,
+                                                        i,
+                                                        i,
+                                                        VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+            if (constraint != VRNA_CONSTRAINT_CONTEXT_NO_REMOVE) {
+              vrna_hc_add_up(fc_AA, i, constraint);
+              vrna_hc_add_up(fc_AA, Alength + i, constraint);
+            }
+          }
+
+          /* base pair constraints next */
+          for (unsigned int i = 1; i <= Alength; ++i) {
+            for (unsigned int k = 1; k < Alength; k++) {
+              unsigned int j = i + k;
+
+              if (j > Alength)
+                break;
+
+              unsigned char constraint = vrna_smx_csr_get(hc_nondefaults,
+                                                          i,
+                                                          j,
+                                                          VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+              if (constraint != VRNA_CONSTRAINT_CONTEXT_NO_REMOVE) {
+                /* insert constraint as plain as possible */
+                vrna_hc_add_bp(fc_AA, i, j, constraint | VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+                vrna_hc_add_bp(fc_AA, Alength + i, Alength + j, constraint | VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+              }
+            }
+          }
+
+          vrna_smx_csr_free(hc_nondefaults);
+        }
+      }
+
+      /* extract soft constraints from concatenated input above, if any */
+      if (vc->sc) {
+        if (vc->sc->up_storage) {
+          for (unsigned int i = 1; i <= Alength; ++i) {
+            vrna_sc_add_up(fc_AA, i, (FLT_OR_DBL)vc->sc->up_storage[i] / 100., VRNA_OPTION_DEFAULT);
+            vrna_sc_add_up(fc_AA, Alength + i, (FLT_OR_DBL)vc->sc->up_storage[i] / 100., VRNA_OPTION_DEFAULT);
+          }
+        }
+
+        if (vc->sc->bp_storage) {
+          for (unsigned int i = 1; i <= Alength; ++i) {
+            if (vc->sc->bp_storage[i]) {
+              for (unsigned int k = 1; k < Alength; k++) {
+                unsigned int j = i + k;
+
+                if (j > Alength)
+                  break;
+
+                int e = get_stored_bp_contributions(vc->sc->bp_storage[i], j);
+
+                if (e != 0) {
+                  vrna_sc_add_bp(fc_AA, i, j, (FLT_OR_DBL)e / 100., VRNA_OPTION_DEFAULT);
+                  vrna_sc_add_bp(fc_AA, Alength + i, Alength + j, (FLT_OR_DBL)e / 100., VRNA_OPTION_DEFAULT);
+                }
+              }
+            }
+          }
+        }
+
+        if (vc->sc->energy_stack) {
+          for (unsigned int i = 1; i <= Alength; ++i) {
+            if (vc->sc->energy_stack[i] != 0) {
+              vrna_sc_add_stack(fc_AA, i, (FLT_OR_DBL)vc->sc->energy_stack[i] / 100., VRNA_OPTION_DEFAULT);
+              vrna_sc_add_stack(fc_AA, Alength + i, (FLT_OR_DBL)vc->sc->energy_stack[i] / 100., VRNA_OPTION_DEFAULT);
+            }
+          }
+        }
+      }
+
       double                mfe_AA = vrna_mfe(fc_AA, ss_AA);
       vrna_mx_mfe_free(fc_AA);
 
@@ -1086,6 +1180,86 @@ process_record(struct record_data *record)
       char                  *ss_BB  = (char *)vrna_alloc(sizeof(char) * (2 * Blength + 1));
       vrna_fold_compound_t  *fc_BB  = vrna_fold_compound(seq_BB, &(opt->md), VRNA_OPTION_DEFAULT);
 
+      /* extract hard constraints from concatenated input above, if any non-defaults */
+      if (vc->hc) {
+        vrna_smx_csr(vrna_uchar) *hc_nondefaults = vrna_hc_nondefaults(vc);
+
+        if (hc_nondefaults) {
+          /* single nucleotide constraints first */
+          for (unsigned int i = Alength + 1; i <= Alength + Blength; ++i) {
+            unsigned char constraint = vrna_smx_csr_get(hc_nondefaults,
+                                                        i,
+                                                        i,
+                                                        VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+            if (constraint != VRNA_CONSTRAINT_CONTEXT_NO_REMOVE) {
+              vrna_hc_add_up(fc_AA, i - Alength, constraint);
+              vrna_hc_add_up(fc_AA, Blength + i - Alength, constraint);
+            }
+          }
+
+          /* base pair constraints next */
+          for (unsigned int i = Alength + 1; i <= Alength + Blength; ++i) {
+            for (unsigned int k = 1; k < Blength; k++) {
+              unsigned int j = i + k;
+
+              if (j > Alength + Blength)
+                break;
+
+              unsigned char constraint = vrna_smx_csr_get(hc_nondefaults,
+                                                          i,
+                                                          j,
+                                                          VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+              if (constraint != VRNA_CONSTRAINT_CONTEXT_NO_REMOVE) {
+                /* insert constraint as plain as possible */
+                vrna_hc_add_bp(fc_AA, i - Alength, j - Alength, constraint | VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+                vrna_hc_add_bp(fc_AA, Blength + i - Alength, Blength + j - Alength, constraint | VRNA_CONSTRAINT_CONTEXT_NO_REMOVE);
+              }
+            }
+          }
+
+          vrna_smx_csr_free(hc_nondefaults);
+        }
+      }
+
+      /* extract soft constraints from concatenated input above, if any */
+      if (vc->sc) {
+        if (vc->sc->up_storage) {
+          for (unsigned int i = Alength + 1; i <= Alength + Blength; ++i) {
+            vrna_sc_add_up(fc_BB, i - Alength, (FLT_OR_DBL)vc->sc->up_storage[i] / 100., VRNA_OPTION_DEFAULT);
+            vrna_sc_add_up(fc_BB, Blength + i - Alength, (FLT_OR_DBL)vc->sc->up_storage[i] / 100., VRNA_OPTION_DEFAULT);
+          }
+        }
+
+        if (vc->sc->bp_storage) {
+          for (unsigned int i = Alength + 1; i <= Alength + Blength; ++i) {
+            if (vc->sc->bp_storage[i]) {
+              for (unsigned int k = 1; k < Blength; k++) {
+                unsigned int j = i + k;
+
+                if (j > Alength + Blength)
+                  break;
+
+                int e = get_stored_bp_contributions(vc->sc->bp_storage[i], j);
+
+                if (e != 0) {
+                  vrna_sc_add_bp(fc_BB, i - Alength, j - Alength, (FLT_OR_DBL)e / 100., VRNA_OPTION_DEFAULT);
+                  vrna_sc_add_bp(fc_BB, Blength + i - Alength, Blength + j - Alength, (FLT_OR_DBL)e / 100., VRNA_OPTION_DEFAULT);
+                }
+              }
+            }
+          }
+        }
+
+        if (vc->sc->energy_stack) {
+          for (unsigned int i = Alength + 1; i <= Alength + Blength; ++i) {
+            if (vc->sc->energy_stack[i] != 0) {
+              vrna_sc_add_stack(fc_BB, i - Alength, (FLT_OR_DBL)vc->sc->energy_stack[i] / 100., VRNA_OPTION_DEFAULT);
+              vrna_sc_add_stack(fc_BB, Blength + i - Alength, (FLT_OR_DBL)vc->sc->energy_stack[i] / 100., VRNA_OPTION_DEFAULT);
+            }
+          }
+        }
+      }
+
       double                mfe_BB = vrna_mfe(fc_BB, ss_BB);
       vrna_mx_mfe_free(fc_BB);
 
@@ -1648,3 +1822,30 @@ read_concentrations(FILE *fp)
   startc[i] = startc[i + 1] = 0;
   return startc;
 }
+
+
+PRIVATE INLINE int
+get_stored_bp_contributions(vrna_sc_bp_storage_t  *container,
+                            unsigned int          j)
+{
+  unsigned int  cnt;
+  int           e;
+
+  e = 0;
+
+  /* go through list of constraints for current position i */
+  for (cnt = 0; container[cnt].interval_start != 0; cnt++) {
+    if (container[cnt].interval_start > j)
+      break; /* only constraints for pairs (i,q) with q > j left */
+
+    if (container[cnt].interval_end < j)
+      continue; /* constraint for pairs (i,q) with q < j */
+
+    /* constraint has interval [p,q] with p <= j <= q */
+    e += container[cnt].e;
+  }
+
+  return e;
+}
+
+
